# frozen_string_literal: true

require "is/inspectable"

module Core
  module Pipeline
    module Actions
      # [public] A pipeline action defined as a block.
      #
      class Block < Action
        include Is::Inspectable

        def initialize(name = nil, before: nil, after: nil, context: nil, &block)
          @block = block

          super(name, before: before, after: after, context: context)
        end

        # [public]
        #
        def finalize(context)
          case @context
          when NilClass
            context.define_method(@name, &@block)
            @name
          else
            if @block.binding.receiver.equal?(@context)
              @block
            else
              @context.define_singleton_method(@name, &@block)
              @context.method(@name).to_proc
            end
          end
        end
      end
    end
  end
end
